-- Can't add wooden roof themes to the existing ones:
-- 1: Currently the themes are on only one line, and there is no Description, so the Edit Window will be too large
-- 2: the textures of wooden roof needs diferent size than the existing texture to fit to the vanilla scale
-- So the roof models are duplicated, and the material related to the theme differs as the type of roof model (tiled or wooden) 

local StoneGateV6 = ...


function registerFbxMaterial(_nameFbxFile, _nameAsset)
    -- Apply building asset processor
    StoneGateV6:registerAssetProcessor("models/woodenRoofs/" .. _nameFbxFile .. ".fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
    })

    -- Register Small Squared Tower materials
    StoneGateV6:registerAssetId("models/woodenRoofs/" .. _nameFbxFile .. ".fbx/Materials/wRedRoof", StoneGateSetPrefix .. "_" .. _nameAsset .. "_RED_WROOF_MATERIAL")
    StoneGateV6:registerAssetId("models/woodenRoofs/" .. _nameFbxFile .. ".fbx/Materials/wBlueRoof", StoneGateSetPrefix .. "_" .. _nameAsset .. "_BLUE_WROOF_MATERIAL")
    StoneGateV6:registerAssetId("models/woodenRoofs/" .. _nameFbxFile .. ".fbx/Materials/wBrownRoof", StoneGateSetPrefix .. "_" .. _nameAsset .. "_BROWN_WROOF_MATERIAL")
    StoneGateV6:registerAssetId("models/woodenRoofs/" .. _nameFbxFile .. ".fbx/Materials/wYellowRoof", StoneGateSetPrefix .. "_" .. _nameAsset .. "_YELLOW_WROOF_MATERIAL")
    StoneGateV6:registerAssetId("models/woodenRoofs/" .. _nameFbxFile .. ".fbx/Materials/wGreenRoof", StoneGateSetPrefix .. "_" .. _nameAsset .. "_GREEN_WROOF_MATERIAL")
end

function registerWoodenRoofPart(_nameFbxFile, _nameFbx, _nameAsset, _buildingPartType, _plankQuantity)
    StoneGateV6:registerAssetId("models/woodenRoofs/" .. _nameFbxFile .. ".fbx/Prefab/" .. _nameFbx .. "Part", StoneGateSetPrefix .. "_" .. _nameAsset .. "_PREFAB")

-- Register the stone gate set decoration roof part
    StoneGateV6:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_" .. _nameAsset .. "_PART",
	Name = StoneGateSetPrefix .. "_" .. _nameAsset .. "_NAME",
	Description = StoneGateSetPrefix .. "_" .. _nameAsset .. "_DESC",
	Category = "EXTENSION",
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = StoneGateSetPrefix .. "_" .. _nameAsset .. "_PREFAB",
                IsPreviewGroundVisible = false
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "PLANK", Quantity = _plankQuantity }
		}
	}
    })

    StoneGateV6:registerPrefabComponent("models/woodenRoofs/" .. _nameFbxFile .. ".fbx/Prefab/" .. _nameFbx .. "Part", {
	DataType = "COMP_BUILDING_PART",
	BuildingPartType = _buildingPartType
    })
end

registerFbxMaterial("wrBasicMain", "MAIN")
registerWoodenRoofPart( "wrBasicMain", "main/Main_WRoofA", "MAIN_WROOF_A", "MAJOR", 7 )
registerWoodenRoofPart( "wrBasicMain", "main/Main_WRoofB", "MAIN_WROOF_B", "MAJOR", 7 )
registerWoodenRoofPart( "wrBasicMain", "main/Main_WRoofD", "MAIN_WROOF_D", "MAJOR", 7 )

registerFbxMaterial("wrBasicKeep", "KEEP")
registerWoodenRoofPart( "wrBasicKeep", "keep/Keep_WRoofA", "KEEP_WROOF_A", "MAJOR", 5 )
registerWoodenRoofPart( "wrBasicKeep", "keep/Keep_WRoofB", "KEEP_WROOF_B", "MAJOR", 5 )
registerWoodenRoofPart( "wrBasicKeep", "keep/Keep_WRoofC", "KEEP_WROOF_C", "MAJOR", 5 )
registerWoodenRoofPart( "wrBasicKeep", "keep/Keep_WRoofD", "KEEP_WROOF_D", "MAJOR", 5 )
registerWoodenRoofPart( "wrBasicKeep", "keep/Keep_WRoofE", "KEEP_WROOF_E", "MAJOR", 5 )
registerWoodenRoofPart( "wrBasicKeep", "keep/Keep_WRoofF", "KEEP_WROOF_F", "MAJOR", 5 )

registerFbxMaterial("wrBasicSmallSquaredTower", "SST")
registerWoodenRoofPart( "wrBasicSmallSquaredTower", "smallSquaredTower/Sst_WRoofA", "SST_WROOF_A", "MAJOR", 3 )
registerWoodenRoofPart( "wrBasicSmallSquaredTower", "smallSquaredTower/Sst_WRoofB", "SST_WROOF_B", "MAJOR", 3 )
registerWoodenRoofPart( "wrBasicSmallSquaredTower", "smallSquaredTower/Sst_WRoofD", "SST_WROOF_D", "MAJOR", 3 )
registerWoodenRoofPart( "wrBasicSmallSquaredTower", "smallSquaredTower/Sst_WRoofE", "SST_WROOF_E", "MAJOR", 3 )
registerWoodenRoofPart( "wrBasicSmallSquaredTower", "smallSquaredTower/Sst_WRoofF", "SST_WROOF_F", "MAJOR", 3 )
registerWoodenRoofPart( "wrBasicSmallSquaredTower", "smallSquaredTower/Sst_WRoofG", "SST_WROOF_G", "MAJOR", 3 )

--registerFbxMaterial("wrPointedRoofSmallSquaredTower", "SST_POINTED")
--registerWoodenRoofPart( "wrPointedRoofSmallSquaredTower", "pointedRoof/Sst_PointedWRoofB", "SST_POINTED_WROOF_B", "MAJOR", 4 )
--registerWoodenRoofPart( "wrPointedRoofSmallSquaredTower", "pointedRoof/Sst_PointedWRoofC", "SST_POINTED_WROOF_C", "MAJOR", 4 )
--registerWoodenRoofPart( "wrPointedRoofSmallSquaredTower", "pointedRoof/Sst_PointedWRoofMA", "SST_POINTED_WROOF_MA", "MAJOR", 5 )
--registerWoodenRoofPart( "wrPointedRoofSmallSquaredTower", "pointedRoof/Sst_PointedWRoofMB", "SST_POINTED_WROOF_MB", "MAJOR", 5 )

registerFbxMaterial("wrBasicOctogonalTower", "OTOWER")
registerWoodenRoofPart( "wrBasicOctogonalTower", "octogonalTower/OctogonalWRoof", "OTOWER_WROOF", "MINOR", 6 )

-- Create default building part registering function 
function registerDefaultBuildingPart(_nodePrefix)
	StoneGateV6:register({
		DataType = "BUILDING_PART",
		Id = _nodePrefix .. "_PART",
		ConstructorData = {
                    DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
                    CoreObjectPrefab = _nodePrefix .. "_PREFAB"
                },
	        Cost = {
		    UpkeepCost = {
			{ Resource = "GOLD", Quantity = 1 }
		    },
		    RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2 }
		    }
	        },
	        EstateSplendor = {
		    { Estate = "KINGDOM", Quantity = 0.1 }
	    }
	})
end

function registerWSmallGatePart(_option, _typeBasement)
        StoneGateV6:register({
	    DataType = "BUILDING_PART",
            Id = StoneGateSetPrefix .. "_W_SGATE" .. _typeBasement .. "_ROOT" .. _option .. "_PART",
            Name = StoneGateSetPrefix .. "_W_SGATE" .. _typeBasement .. "_ROOT" .. _option .. "_NAME",
            Description = StoneGateSetPrefix .. "_W_SGATE" .. _typeBasement .. "_ROOT" .. _option .. "_DESC",
            Category = "CORE",
            BuildingZone = { ZoneEntryList = {
                { Polygon = polygon.createRectangle( { 0.1, 0.1 }, { 0, 3.5 } ),
                  Type = { DEFAULT = true, NAVIGABLE = false, GRASS_CLEAR = true } }
            }},
	    ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER",
		CoreObjectPrefab = "STONE_GATE_V5_SGATE_ROOT_PREFAB",
		EndPart = StoneGateSetPrefix .. "_W_SGATE_TOP" .. _option .. "_PART",
		FillerList = {
			"STONE_GATE_V5_SGATE_TILING2_PART",
			"STONE_GATE_V5_SGATE_TILING3_PART"
                },
		BasementFillerList = {
			"STONE_GATE_V5_SGATE" .. _typeBasement .. "_TILING1_PART",
			"STONE_GATE_V5_BGATE_TILING1_PART",
			"STONE_GATE_V5_BGATE_TILING1_PART",
			"STONE_GATE_V5_BGATE_TILING1_PART"
         	},
		MinimumScale = 0,
		IsVertical = true,
                IsPreviewGroundVisible = false
	    },
	    Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2 }
		}
	    },
	    EstateSplendor = {
		{ Estate = "KINGDOM", Quantity = 1 }
	    }
        })
end

function registerWSmallGate(_option)
    -- Register Small Gate prefab nodes
    StoneGateV6:registerAssetId("models/woodenRoofs/wrBasicSmallGate.fbx/Prefab/smallGate/step_3_W_SGate_Top" .. _option .. "Part", StoneGateSetPrefix .. "_W_SGATE_TOP" .. _option .. "_PREFAB")
    registerDefaultBuildingPart(StoneGateSetPrefix .. "_W_SGATE_TOP" .. _option)
    registerWSmallGatePart(_option, "") -- Register the stone gate set small gate root part
    registerWSmallGatePart(_option, "_EXT") -- Register the stone gate set small gate extension root part
end

registerFbxMaterial("wrBasicSmallGate", "SGATE")
registerWSmallGate("") -- with stone floor
registerWSmallGate("_P") -- with wooden floor

